@@ -16,16 +16,16 @@
 
 # Author: Cyril Jaquier
 # 
-# $Revision: 553 $
+# $Revision: 638 $
 
 __author__ = "Cyril Jaquier"
-__version__ = "$Revision: 553 $"
-__date__ = "$Date: 2007-02-26 00:53:22 +0100 (Mon, 26 Feb 2007) $"
+__version__ = "$Revision: 638 $"
+__date__ = "$Date: 2007-12-17 21:00:36 +0100 (Mon, 17 Dec 2007) $"
 __copyright__ = "Copyright (c) 2004 Cyril Jaquier"
 __license__ = "GPL"
 
 from faildata import FailData
-from failticket import FailTicket
+from ticket import FailTicket
 from threading import Lock
 import logging
 
